% Chudik, Pesaran and Yang (2016)
% 
% Data generating process
% 
% Y(i,t) = muY(i) + delta(t) + lambdaY * Y(i,t-1) 
%           + [1 - lambdaY] * beta * X(i,t) + U(i,t).
% X(i,t) = [1 - lambdaX] * muX(i) + [1 - lambdaX] * kappaX * Y(i,t-1)
%           + lambdaX * X(i,t-1) + v(i,t)
%
% beta = 0.5, lambdaX = 0.25, delta(t) = 0.025 * t,
% muX(i) ~ i.i.d. N(1, 1), muY(i) = muX(i) + etaY(i), etaY(i) ~ i.i.d. N(1, 1),
% v(i,t) ~ i.i.d. N(0, sigmav^2(i)), sigmav^2(i) = 0.5 + 0.25 * etav^2(i),
%           etav^2(i) ~ i.i.d. chi^2(2),
% u(i,t) ~ i.i.d. N(0, sigmau^2(i)), sigmau^2(i) = 0.5 + 0.25 * etau^2(i),
%           etau^2(i) ~ i.i.d. chi^2(2),
% lambdaY = 0, 
% kappaX = 0, 0.2 or 0.4.

clc
clear
close all

tic

parpool

% Replications
R = 2000;

% Parameters
betaY = 0.5;
lambdaX = 0.25;

g1Y = 0.025;
g2Y = 0;
lambdaY = 0;
kappaX = 0.2;


% phiY and bY

phiY = lambdaY - 1;
bY = (1 - lambdaY ) * betaY;

% Sample Size
NN = [30; 60; 100; 200; 500; 1000];
TT = [30; 60; 100; 200];

DN = length(NN);
DT = length(TT);

maxN = NN(DN);
maxT = TT(DT);

% Burn-in
B = 100;

% Results

% r: robust, d: Hashem's formula

COEFFEbetaY = zeros(R, DN, DT);
STDFEbetaY = zeros(R, DN, DT);
BIASFEbetaY = zeros(DN, DT);
MSEFEbetaY = zeros(DN, DT);
SIZEFEbetaY = zeros(DN, DT);

COEFFEbetaYJ = zeros(R, DN, DT);
STDFEbetaYJ = zeros(R, DN, DT);
BIASFEbetaYJ = zeros(DN, DT);
MSEFEbetaYJ = zeros(DN, DT);
SIZEFEbetaYJ = zeros(DN, DT);

COEFFETEbetaY = zeros(R, DN, DT);
STDFETEbetaY = zeros(R, DN, DT);
BIASFETEbetaY = zeros(DN, DT);
MSEFETEbetaY = zeros(DN, DT);
SIZEFETEbetaY = zeros(DN, DT);

COEFFETEbetaYJ = zeros(R, DN, DT);
STDFETEbetaYJ = zeros(R, DN, DT);
BIASFETEbetaYJ = zeros(DN, DT);
MSEFETEbetaYJ = zeros(DN, DT);
SIZEFETEbetaYJ = zeros(DN, DT);


for r = 1:R
    
    N = maxN;
    T = B + maxT;
    
    % Parameters
    
    LambdaY = ones(N, 1) * lambdaY;
    BetaY = ones(N, 1) * betaY;
    G1Y = ones(N, 1) * g1Y;
    G2Y = ones(N, 1) * g2Y;
    
    SigmaU2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaU = SigmaU2.^0.5;
    
    LambdaX = ones(N, 1) * lambdaX;
    KappaX = ones(N, 1) * kappaX;
    
    CX = ones(N, 1) + randn(N, 1);
    CY = CX + ones(N, 1) + randn(N, 1);
    
    SigmaV2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaV = SigmaV2.^0.5;
    
    % Errors
    
    UY = randn(N, T) .* (SigmaU * ones(1, T));
    VX = randn(N, T) .* (SigmaV * ones(1, T));
    
    % Generating Data
    
    LX = zeros(N, 1);
    LY = zeros(N, 1);
    
    XX = zeros(N, T);
    YY = zeros(N, T);

    for t = 1:T

        X = (ones(N, 1) - LambdaX) .* CX + LambdaX .* LX...
            + (ones(N, 1) - LambdaX) .* KappaX .* LY + VX(:, t);
        
        Y = CY + G1Y * (t-B) + G2Y * (t-B)^2 ...
            + LambdaY .* LY + (ones(N, 1) - LambdaY) .* BetaY .* X + UY(:, t);

        XX(:, t) = X;
        YY(:, t) = Y;

        LX = X;
        LY = Y;

    end
    
    YY = YY(:, (B+1):T);
    XX = XX(:, (B+1):T);
    
    % Estimation
    
    for dn = 1:DN
        
        N = NN(dn);
        
        for dt = 1:DT
            
            T = TT(dt);
            
            Y = YY(1:N, 1:T);
            X = XX(1:N, 1:T);
            
            % FE Estimation
            
            Ym = Y - mean(Y, 2) * ones(1, T);
            Xm = X - mean(X, 2) * ones(1, T);
            Tm = (1:T) - mean((1:T)) * ones(1, T);
            
            Ymtau = Y - ones(N, 1) * mean(Y, 1) - mean(Y, 2) * ones(1, T) + ones(N, T) * mean(mean(Y));
            Xmtau = X - ones(N, 1) * mean(X, 1) - mean(X, 2) * ones(1, T) + ones(N, T) * mean(mean(X));
            
            WMW = zeros(2, 2);
            WMDY = zeros(2, 1);
            
            WMWtau = 0;
            WMDYtau = 0;
            
            for i = 1:N
                
                YYm = Ym(i, :)';
                Wm = [Xm(i, :)', Tm'];
                WMW = WMW + Wm' * Wm;
                WMDY = WMDY + Wm' * YYm;
                
                YYmtau = Ymtau(i, :)';
                Wmtau = Xmtau(i, :)';
                WMWtau = WMWtau + Wmtau' * Wmtau;
                WMDYtau = WMDYtau + Wmtau' * YYmtau;
                
            end
            
            invWMW = WMW^(-1);
            FEest = invWMW * WMDY;
            BetaYFE = FEest(1);
            
            COEFFEbetaY(r, dn, dt) = BetaYFE;
            BIASFEbetaY(dn, dt) = BIASFEbetaY(dn, dt) + 100 * (BetaYFE - betaY) / R;
            MSEFEbetaY(dn, dt) = MSEFEbetaY(dn, dt) + 10000 * (BetaYFE - betaY)^2 / R;
            
            invWMWtau = WMWtau^(-1);
            FETEest = invWMWtau * WMDYtau;
            BetaYFETE = FETEest(1);
            
            COEFFETEbetaY(r, dn, dt) = BetaYFETE;
            BIASFETEbetaY(dn, dt) = BIASFETEbetaY(dn, dt) + 100 * (BetaYFETE - betaY) / R;
            MSEFETEbetaY(dn, dt) = MSEFETEbetaY(dn, dt) + 10000 * (BetaYFETE - betaY)^2 / R;
            
            % Jackknife
            
            Yma = Y(:, 1:T/2) - mean(Y(:, 1:T/2), 2) * ones(1, T/2);
            Ymb = Y(:, T/2+1:T) - mean(Y(:, T/2+1:T), 2) * ones(1, T/2);
            Xma = X(:, 1:T/2) - mean(X(:, 1:T/2), 2) * ones(1, T/2);
            Xmb = X(:, T/2+1:T) - mean(X(:, T/2+1:T), 2) * ones(1, T/2);
            Tma = (1:T/2) - mean((1:T/2)) * ones(1, T/2);
            Tmb = (T/2+1:T) - mean((T/2+1:T)) * ones(1, T/2);
            
            Ymtaua = Y(:, 1:T/2) - ones(N, 1) * mean(Y(:, 1:T/2), 1) - mean(Y(:, 1:T/2), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(Y(:, 1:T/2)));
            Ymtaub = Y(:, T/2+1:T) - ones(N, 1) * mean(Y(:, T/2+1:T), 1) - mean(Y(:, T/2+1:T), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(Y(:, T/2+1:T)));
            Xmtaua = X(:, 1:T/2) - ones(N, 1) * mean(X(:, 1:T/2), 1) - mean(X(:, 1:T/2), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(X(:, 1:T/2)));
            Xmtaub = X(:, T/2+1:T) - ones(N, 1) * mean(X(:, T/2+1:T), 1) - mean(X(:, T/2+1:T), 2) * ones(1, T/2) + ones(N, T/2) * mean(mean(X(:, T/2+1:T)));
            
            WMWa = zeros(2, 2);
            WMDYa = zeros(2, 1);
            WMWb = zeros(2, 2);
            WMDYb = zeros(2, 1);
            
            WMWtaua = 0;
            WMDYtaua = 0;
            WMWtaub = 0;
            WMDYtaub = 0;
            
            for i = 1:N
                
                YYma = Yma(i, :)';
                Wma = [Xma(i, :)', Tma'];
                WMWa = WMWa + Wma' * Wma;
                WMDYa = WMDYa + Wma' * YYma;
                
                YYmb = Ymb(i, :)';
                Wmb = [Xmb(i, :)', Tmb'];
                WMWb = WMWb + Wmb' * Wmb;
                WMDYb = WMDYb + Wmb' * YYmb;
                
                YYmtaua = Ymtaua(i, :)';
                Wmtaua = Xmtaua(i, :)';
                WMWtaua = WMWtaua + Wmtaua' * Wmtaua;
                WMDYtaua = WMDYtaua + Wmtaua' * YYmtaua;
                
                YYmtaub = Ymtaub(i, :)';
                Wmtaub = Xmtaub(i, :)';
                WMWtaub = WMWtaub + Wmtaub' * Wmtaub;
                WMDYtaub = WMDYtaub + Wmtaub' * YYmtaub;
                
            end
            
            invWMWa = WMWa^(-1);
            FEesta = invWMWa * WMDYa;
            invWMWb = WMWb^(-1);
            FEestb = invWMWb * WMDYb;
            FEestJ = 2 * FEest - 0.5 * FEesta - 0.5 * FEestb;
            BetaYFEJ = FEestJ(1);
            
            COEFFEbetaYJ(r, dn, dt) = BetaYFEJ;
            BIASFEbetaYJ(dn, dt) = BIASFEbetaYJ(dn, dt) + 100 * (BetaYFEJ - betaY) / R;
            MSEFEbetaYJ(dn, dt) = MSEFEbetaYJ(dn, dt) + 10000 * (BetaYFEJ - betaY)^2 / R;
            
            invWMWtaua = WMWtaua^(-1);
            FETEesta = invWMWtaua * WMDYtaua;
            invWMWtaub = WMWtaub^(-1);
            FETEestb = invWMWtaub * WMDYtaub;
            FETEestJ = 2 * FETEest - 0.5 * FETEesta - 0.5 * FETEestb;
            BetaYFETEJ = FETEestJ(1);
            
            COEFFETEbetaYJ(r, dn, dt) = BetaYFETEJ;
            BIASFETEbetaYJ(dn, dt) = BIASFETEbetaYJ(dn, dt) + 100 * (BetaYFETEJ - betaY) / R;
            MSEFETEbetaYJ(dn, dt) = MSEFETEbetaYJ(dn, dt) + 10000 * (BetaYFETEJ - betaY)^2 / R;
            
            % Variance
            
            WuuW = zeros(2, 2);
            WuuWJ = zeros(2, 2);
            
            WuuWtau = 0;
            WuuWtauJ = 0;
            
            Ymma = Y(:, 1:T/2) - 2 * mean(Y, 2) * ones(1, T/2) + mean(Y(:, 1:T/2), 2) * ones(1, T/2);
            Ymmb = Y(:, T/2+1:T) - 2 * mean(Y, 2) * ones(1, T/2) + mean(Y(:, T/2+1:T), 2) * ones(1, T/2);
            Ymm = [Ymma, Ymmb];
            Xmma = X(:, 1:T/2) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, 1:T/2), 2) * ones(1, T/2);
            Xmmb = X(:, T/2+1:T) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, T/2+1:T), 2) * ones(1, T/2);
            Xmm = [Xmma, Xmmb];
            Tmma = (1:T/2) - 2 * mean((1:T)) * ones(1, T/2) + mean((1:T/2)) * ones(1, T/2);
            Tmmb = (T/2+1:T) - 2 * mean((1:T)) * ones(1, T/2) + mean((T/2+1:T)) * ones(1, T/2);
            Tmm = [Tmma, Tmmb];
            
            Ymmtaua = Y(:, 1:T/2) - ones(N, 1) * mean(Y(:, 1:T/2), 1) - 2 * mean(Y, 2) * ones(1, T/2) + mean(Y(:, 1:T/2), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(Y)) - ones(N, T/2) * mean(mean(Y(:, 1:T/2)));
            Ymmtaub = Y(:, T/2+1:T) - ones(N, 1) * mean(Y(:, T/2+1:T), 1) - 2 * mean(Y, 2) * ones(1, T/2) + mean(Y(:, T/2+1:T), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(Y)) - ones(N, T/2) * mean(mean(Y(:, T/2+1:T)));
            Ymmtau = [Ymmtaua, Ymmtaub];
            Xmmtaua = X(:, 1:T/2) - ones(N, 1) * mean(X(:, 1:T/2), 1) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, 1:T/2), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(X)) - ones(N, T/2) * mean(mean(X(:, 1:T/2)));
            Xmmtaub = X(:, T/2+1:T) - ones(N, 1) * mean(X(:, T/2+1:T), 1) - 2 * mean(X, 2) * ones(1, T/2) + mean(X(:, T/2+1:T), 2) * ones(1, T/2)...
                + 2 * ones(N, T/2) * mean(mean(X)) - ones(N, T/2) * mean(mean(X(:, T/2+1:T)));
            Xmmtau = [Xmmtaua, Xmmtaub];
            
            for i = 1:N
                
                YYm = Ym(i, :)';
                Wm = [Xm(i, :)', Tm'];
                Wmm = [Xmm(i, :)', Tmm'];
                uFE = YYm - Wm * FEest;
                uFEJ = YYm - Wm * FEestJ;
                
                for t = 1:T
                    
                    WuuW = WuuW + uFE(t)^2 * Wm(t, :)'* Wm(t, :);
                    WuuWJ = WuuWJ + uFEJ(t)^2 * Wmm(t, :)'* Wmm(t, :);
                    
                end
                
                YYmtau = Ymtau(i, :)';
                Wmtau = Xmtau(i, :)';
                Wmmtau = Xmmtau(i, :)';
                uFETE = YYmtau - Wmtau * FETEest;
                uFETEJ = YYmtau - Wmtau * FETEestJ;
                
                for t = 1:T
                    
                    WuuWtau = WuuWtau + uFETE(t)^2 * Wmtau(t, :)' * Wmtau(t, :);
                    WuuWtauJ = WuuWtauJ + uFETEJ(t)^2 * Wmmtau(t, :)' * Wmmtau(t, :);
                    
                end
                
            end
            
            AvarFE = invWMW * WuuW * invWMW;
            BetaYFEstd = sqrt(AvarFE(1, 1));
                        
            AvarFEJ = invWMW * WuuWJ * invWMW;
            BetaYFEstdJ = sqrt(AvarFEJ(1, 1));
            
            AvarFETE = invWMWtau * WuuWtau * invWMWtau;
            BetaYFETEstd = sqrt(AvarFETE(1, 1));
                        
            AvarFETEJ = invWMWtau * WuuWtauJ * invWMWtau;
            BetaYFETEstdJ = sqrt(AvarFETEJ(1, 1));
            
            STDFEbetaY(r, dn, dt) = BetaYFEstd;
            SIZEFEbetaY(dn, dt) = SIZEFEbetaY(dn, dt) + 100 * (abs((BetaYFE - betaY) / BetaYFEstd) > 1.96) / R;
            
            STDFEbetaYJ(r, dn, dt) = BetaYFEstdJ;
            SIZEFEbetaYJ(dn, dt) = SIZEFEbetaYJ(dn, dt) + 100 * (abs((BetaYFEJ - betaY) / BetaYFEstdJ) > 1.96) / R;
            
            STDFETEbetaY(r, dn, dt) = BetaYFETEstd;
            SIZEFETEbetaY(dn, dt) = SIZEFETEbetaY(dn, dt) + 100 * (abs((BetaYFETE - betaY) / BetaYFETEstd) > 1.96) / R;
            
            STDFETEbetaYJ(r, dn, dt) = BetaYFETEstdJ;
            SIZEFETEbetaYJ(dn, dt) = SIZEFETEbetaYJ(dn, dt) + 100 * (abs((BetaYFETEJ - betaY) / BetaYFETEstdJ) > 1.96) / R;
            
        end
        
    end
    
    r
    toc
    
end

clearvars -except savefile g1Y g2Y lambdaY kappaX...
    COEFFEbetaY STDFEbetaY BIASFEbetaY MSEFEbetaY SIZEFEbetaY...
    COEFFEbetaYJ STDFEbetaYJ BIASFEbetaYJ MSEFEbetaYJ SIZEFEbetaYJ...
    COEFFETEbetaY STDFETEbetaY BIASFETEbetaY MSEFETEbetaY SIZEFETEbetaY...
    COEFFETEbetaYJ STDFETEbetaYJ BIASFETEbetaYJ MSEFETEbetaYJ SIZEFETEbetaYJ

RMSEFEbetaY = MSEFEbetaY.^0.5;
RMSEFEbetaYJ = MSEFEbetaYJ.^0.5;
RMSEFETEbetaY = MSEFETEbetaY.^0.5;
RMSEFETEbetaYJ = MSEFETEbetaYJ.^0.5;

toc

g1Y
g2Y
lambdaY
kappaX

BIASFEbetaY
BIASFEbetaYJ
BIASFETEbetaY
BIASFETEbetaYJ

RMSEFEbetaY
RMSEFEbetaYJ
RMSEFETEbetaY
RMSEFETEbetaYJ

SIZEFEbetaY
SIZEFEbetaYJ
SIZEFETEbetaY
SIZEFETEbetaYJ

delete(gcp)

toc