% Chudik, Pesaran and Yang (2016)
% 
% Data generating process
% 
% Y(i,t) = muY(i) + delta(t) + lambdaY * Y(i,t-1) 
%           + [1 - lambdaY] * beta * X(i,t) + U(i,t).
% X(i,t) = [1 - lambdaX] * muX(i) + [1 - lambdaX] * kappaX * Y(i,t-1)
%           + lambdaX * X(i,t-1) + v(i,t)
%
% beta = 0.5, lambdaX = 0.25, delta(t) = 0, 0.025 * t or 0.025 * t - 0.001 * t^2
% muX(i) ~ i.i.d. N(1, 1), muY(i) = muX(i) + etaY(i), etaY(i) ~ i.i.d. N(1, 1),
% v(i,t) ~ i.i.d. N(0, sigmav^2(i)), sigmav^2(i) = 0.5 + 0.25 * etav^2(i),
%           etav^2(i) ~ i.i.d. chi^2(2),
% u(i,t) ~ i.i.d. N(0, sigmau^2(i)), sigmau^2(i) = 0.5 + 0.25 * etau^2(i),
%           etau^2(i) ~ i.i.d. chi^2(2),
% lambdaY = 0.4 or 0.8, 
% kappaX = 0, 0.2 or 0.4.

clc
clear
close all

tic

% Replications
R = 2000;

% Parameters
betaY = 0.5;
lambdaX = 0.25;

g1Y = 0;
g2Y = 0;
lambdaY = 0.4;
kappaX = 0.4;

unbalance = 1; % unbalanced panel

% phiY and bY

phiY = lambdaY - 1;
bY = (1 - lambdaY ) * betaY;

% Sample Size
NN = [30; 60; 100; 200; 500; 1000];
TT = [30; 60; 100; 200];

DN = length(NN);
DT = length(TT);

maxN = NN(DN);
maxT = TT(DT);

% Burn-in
B = 100;

% Results

COEFFEbetaY = zeros(R, DN, DT);
STDFEbetaY = zeros(R, DN, DT);
BIASFEbetaY = zeros(DN, DT);
MSEFEbetaY = zeros(DN, DT);
SIZEFEbetaY = zeros(DN, DT);

COEFFEbetaYJ = zeros(R, DN, DT);
STDFEbetaYJ = zeros(R, DN, DT);
BIASFEbetaYJ = zeros(DN, DT);
MSEFEbetaYJ = zeros(DN, DT);
SIZEFEbetaYJ = zeros(DN, DT);

COEFFEphiY = zeros(R, DN, DT);
STDFEphiY = zeros(R, DN, DT);
BIASFEphiY = zeros(DN, DT);
MSEFEphiY = zeros(DN, DT);
SIZEFEphiY = zeros(DN, DT);

COEFFEphiYJ = zeros(R, DN, DT);
STDFEphiYJ = zeros(R, DN, DT);
BIASFEphiYJ = zeros(DN, DT);
MSEFEphiYJ = zeros(DN, DT);
SIZEFEphiYJ = zeros(DN, DT);

COEFFEbY = zeros(R, DN, DT);
STDFEbY = zeros(R, DN, DT);
BIASFEbY = zeros(DN, DT);
MSEFEbY = zeros(DN, DT);
SIZEFEbY = zeros(DN, DT);

COEFFEbYJ = zeros(R, DN, DT);
STDFEbYJ = zeros(R, DN, DT);
BIASFEbYJ = zeros(DN, DT);
MSEFEbYJ = zeros(DN, DT);
SIZEFEbYJ = zeros(DN, DT);

for r = 1:R
    
    N = maxN;
    T = B + maxT;
    
    % Parameters
    
    LambdaY = ones(N, 1) * lambdaY;
    BetaY = ones(N, 1) * betaY;
    G1Y = ones(N, 1) * g1Y;
    G2Y = ones(N, 1) * g2Y;
    
    SigmaU2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaU = SigmaU2.^0.5;
    
    LambdaX = ones(N, 1) * lambdaX;
    KappaX = ones(N, 1) * kappaX;
    
    CX = ones(N, 1) + randn(N, 1);
    CY = CX + ones(N, 1) + randn(N, 1);
    
    SigmaV2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaV = SigmaV2.^0.5;
    
    % Errors
    
    UY = randn(N, T) .* (SigmaU * ones(1, T));
    VX = randn(N, T) .* (SigmaV * ones(1, T));
    
    % Generating Data
    
    LX = zeros(N, 1);
    LY = zeros(N, 1);
    
    XX = zeros(N, T);
    YY = zeros(N, T);

    for t = 1:T

        X = (ones(N, 1) - LambdaX) .* CX + LambdaX .* LX...
            + (ones(N, 1) - LambdaX) .* KappaX .* LY...
            + VX(:, t);

        Y = CY + G1Y * (t-B) + G2Y * (t-B)^2 ...
            + LambdaY .* LY + (ones(N, 1) - LambdaY) .* BetaY .* X + UY(:, t);

        XX(:, t) = X;
        YY(:, t) = Y;

        LX = X;
        LY = Y;

    end
    
    LYY = YY(:, B:(T-1));
    YY = YY(:, (B+1):T);
    DYY = YY - LYY;
    XX = XX(:, (B+1):T);
    
    % Estimation
    
    for dn = 1:DN
        
        N = NN(dn);
        
        for dt = 1:DT
            
            T = TT(dt);
            
            Y = YY(1:N, 1:T);
            LY = LYY(1:N, 1:T);
            DY = DYY(1:N, 1:T);
            X = XX(1:N, 1:T);
    
            Trend = ones(N, 1) * (1:T);
            
            UnBd = ones(N, T);
            
            if unbalance == 1;
                
                UnBd(1:floor(N/4), 1:floor(T/5)) = zeros(floor(N/4), floor(T/5));
                UnBd(1:floor(N/4), ceil(4*T/5)+1:T) = zeros(floor(N/4), floor(T/5));
                UnBd(floor(N/4)+1:floor(N/2), 1:floor(T/3)) = zeros(floor(N/2)-floor(N/4), floor(T/3));
                UnBd(floor(N/4)+1:floor(N/2), ceil(2*T/3)+1:T) = zeros(floor(N/2)-floor(N/4), floor(T/3));
                
            end
            
            Y = Y .* UnBd;
            LY = LY .* UnBd;
            DY = DY .* UnBd;
            X = X .* UnBd;
            Trend = Trend .* UnBd;
            
            Ya = Y(:, 1:T/2);
            LYa = LY(:, 1:T/2);
            DYa = DY(:, 1:T/2);
            Xa = X(:, 1:T/2);
            Trenda = Trend(:, 1:T/2);
            
            Yb = Y(:, T/2+1:T);
            LYb = LY(:, T/2+1:T);
            DYb = DY(:, T/2+1:T);
            Xb = X(:, T/2+1:T);
            Trendb = Trend(:, T/2+1:T);
            
            % FE Estimation
            
            Ybari = zeros(N, 1);
            LYbari = zeros(N, 1);
            DYbari = zeros(N, 1);
            Xbari = zeros(N, 1);
            Trendbari = zeros(N, 1);
            
            Ybaria = zeros(N, 1);
            LYbaria = zeros(N, 1);
            DYbaria = zeros(N, 1);
            Xbaria = zeros(N, 1);
            Trendbaria = zeros(N, 1);
            
            Ybarib = zeros(N, 1);
            LYbarib = zeros(N, 1);
            DYbarib = zeros(N, 1);
            Xbarib = zeros(N, 1);
            Trendbarib = zeros(N, 1);
            
            for i = 1:N
                
                y = Y(i, :)';
                Ybari(i) = mean(y(y~=0));
                y = LY(i, :)';
                LYbari(i) = mean(y(y~=0));
                y = DY(i, :)';
                DYbari(i) = mean(y(y~=0));
                y = X(i, :)';
                Xbari(i) = mean(y(y~=0));
                y = Trend(i, :)';
                Trendbari(i) = mean(y(y~=0));
                
                y = Ya(i, :)';
                Ybaria(i) = mean(y(y~=0));
                y = LYa(i, :)';
                LYbaria(i) = mean(y(y~=0));
                y = DYa(i, :)';
                DYbaria(i) = mean(y(y~=0));
                y = Xa(i, :)';
                Xbaria(i) = mean(y(y~=0));
                y = Trenda(i, :)';
                Trendbaria(i) = mean(y(y~=0));
                
                y = Yb(i, :)';
                Ybarib(i) = mean(y(y~=0));
                y = LYb(i, :)';
                LYbarib(i) = mean(y(y~=0));
                y = DYb(i, :)';
                DYbarib(i) = mean(y(y~=0));
                y = Xb(i, :)';
                Xbarib(i) = mean(y(y~=0));
                y = Trendb(i, :)';
                Trendbarib(i) = mean(y(y~=0));
                
            end
            
            Ybari = Ybari * ones(1, T) .* UnBd;
            LYbari = LYbari * ones(1, T) .* UnBd;
            DYbari = DYbari * ones(1, T) .* UnBd;
            Xbari = Xbari * ones(1, T) .* UnBd;
            Trendbari = Trendbari * ones(1, T) .* UnBd;
            
            Ybaria = Ybaria * ones(1, T/2) .* UnBd(:, 1:T/2);
            LYbaria = LYbaria * ones(1, T/2) .* UnBd(:, 1:T/2);
            DYbaria = DYbaria * ones(1, T/2) .* UnBd(:, 1:T/2);
            Xbaria = Xbaria * ones(1, T/2) .* UnBd(:, 1:T/2);
            Trendbaria = Trendbaria * ones(1, T/2) .* UnBd(:, 1:T/2);
            
            Ybarib = Ybarib * ones(1, T/2) .* UnBd(:, T/2+1:T);
            LYbarib = LYbarib * ones(1, T/2) .* UnBd(:, T/2+1:T);
            DYbarib = DYbarib * ones(1, T/2) .* UnBd(:, T/2+1:T);
            Xbarib = Xbarib * ones(1, T/2) .* UnBd(:, T/2+1:T);
            Trendbarib = Trendbarib * ones(1, T/2) .* UnBd(:, T/2+1:T);
            
            Ybariab = [Ybaria, Ybarib];
            LYbariab = [LYbaria, LYbarib];
            DYbariab = [DYbaria, DYbarib];
            Xbariab = [Xbaria, Xbarib];
            Trendbariab = [Trendbaria, Trendbarib];
            
            WMW = zeros(3, 3);
            WMDY = zeros(3, 1);
            
            for i = 1:N
                
                dYm = DY(i, :)' - DYbari(i, :)';
                LYm = LY(i, :)' - LYbari(i, :)';
                Xm = X(i, :)' - Xbari(i, :)';
                Trendm = Trend(i, :)' - Trendbari(i, :)';
                Wm = [LYm, Xm, Trendm];
                WMW = WMW + Wm' * Wm;
                WMDY = WMDY + Wm' * dYm;
                
            end
            
            invWMW = WMW^(-1);
            FEest = invWMW * WMDY;
            BetaYFE = - FEest(2) / FEest(1);
            
            COEFFEbetaY(r, dn, dt) = BetaYFE;
            BIASFEbetaY(dn, dt) = BIASFEbetaY(dn, dt) + 100 * (BetaYFE - betaY) / R;
            MSEFEbetaY(dn, dt) = MSEFEbetaY(dn, dt) + 10000 * (BetaYFE - betaY)^2 / R;
            
            COEFFEphiY(r, dn, dt) = FEest(1);
            BIASFEphiY(dn, dt) = BIASFEphiY(dn, dt) + 100 * (FEest(1) - phiY) / R;
            MSEFEphiY(dn, dt) = MSEFEphiY(dn, dt) + 10000 * (FEest(1) - phiY)^2 / R;
            
            COEFFEbY(r, dn, dt) = FEest(2);
            BIASFEbY(dn, dt) = BIASFEbY(dn, dt) + 100 * (FEest(2) - bY) / R;
            MSEFEbY(dn, dt) = MSEFEbY(dn, dt) + 10000 * (FEest(2) - bY)^2 / R;
            
            % Jackknife
            
            WMWa = zeros(3, 3);
            WMDYa = zeros(3, 1);
            WMWb = zeros(3, 3);
            WMDYb = zeros(3, 1);
            
            for i = 1:N
                
                dYma = DYa(i, :)' - DYbaria(i, :)';
                LYma = LYa(i, :)' - LYbaria(i, :)';
                Xma = Xa(i, :)' - Xbaria(i, :)';
                Trendma = Trenda(i, :)' - Trendbaria(i, :)';
                Wma = [LYma, Xma, Trendma];
                WMWa = WMWa + Wma' * Wma;
                WMDYa = WMDYa + Wma' * dYma;
                
                dYmb = DYb(i, :)' - DYbarib(i, :)';
                LYmb = LYb(i, :)' - LYbarib(i, :)';
                Xmb = Xb(i, :)' - Xbarib(i, :)';
                Trendmb = Trendb(i, :)' - Trendbarib(i, :)';
                Wmb = [LYmb, Xmb, Trendmb];
                WMWb = WMWb + Wmb' * Wmb;
                WMDYb = WMDYb + Wmb' * dYmb;
                
            end
            
            invWMWa = WMWa^(-1);
            FEesta = invWMWa * WMDYa;
            invWMWb = WMWb^(-1);
            FEestb = invWMWb * WMDYb;
            FEestJ = 2 * FEest - 0.5 * FEesta - 0.5 * FEestb;
            BetaYFEJ = - FEestJ(2) / FEestJ(1);
            
            COEFFEbetaYJ(r, dn, dt) = BetaYFEJ;
            BIASFEbetaYJ(dn, dt) = BIASFEbetaYJ(dn, dt) + 100 * (BetaYFEJ - betaY) / R;
            MSEFEbetaYJ(dn, dt) = MSEFEbetaYJ(dn, dt) + 10000 * (BetaYFEJ - betaY)^2 / R;
            
            COEFFEphiYJ(r, dn, dt) = FEestJ(1);
            BIASFEphiYJ(dn, dt) = BIASFEphiYJ(dn, dt) + 100 * (FEestJ(1) - phiY) / R;
            MSEFEphiYJ(dn, dt) = MSEFEphiYJ(dn, dt) + 10000 * (FEestJ(1) - phiY)^2 / R;
            
            COEFFEbYJ(r, dn, dt) = FEestJ(2);
            BIASFEbYJ(dn, dt) = BIASFEbYJ(dn, dt) + 100 * (FEestJ(2) - bY) / R;
            MSEFEbYJ(dn, dt) = MSEFEbYJ(dn, dt) + 10000 * (FEestJ(2) - bY)^2 / R;
            
            % Variance
            
            WuuW = zeros(3, 3);
            WuuWJ = zeros(3, 3);
            
            for i = 1:N
                
                dYm = DY(i, :)' - DYbari(i, :)';
                LYm = LY(i, :)' - LYbari(i, :)';
                Xm = X(i, :)' - Xbari(i, :)';
                Trendm = Trend(i, :)' - Trendbari(i, :)';
                Wm = [LYm, Xm, Trendm];
                LYmm = LY(i, :)' - 2 * LYbari(i, :)' + LYbariab(i, :)';
                Xmm = X(i, :)' - 2 * Xbari(i, :)' + Xbariab(i, :)';
                Trendmm = Trend(i, :)' - 2 * Trendbari(i, :)' + Trendbariab(i, :)';
                Wmm = [LYmm, Xmm, Trendmm];
                uFE = dYm - Wm * FEest;
                uFEJ = dYm - Wm * FEestJ;
                
                for t = 1:T
                    
                    WuuW = WuuW + uFE(t)^2 * Wm(t, :)'* Wm(t, :);
                    WuuWJ = WuuWJ + uFEJ(t)^2 * Wmm(t, :)'* Wmm(t, :);
                    
                end
                
            end
            
            AvarFE = invWMW * WuuW * invWMW;
            BetaYFEstd = sqrt([FEest(2) / FEest(1)^2, -1 / FEest(1)] * AvarFE(1:2, 1:2) * [FEest(2) / FEest(1)^2, -1 / FEest(1)]');
                        
            AvarFEJ = invWMW * WuuWJ * invWMW;
            BetaYFEstdJ = sqrt([FEestJ(2) / FEestJ(1)^2, -1 / FEestJ(1)] * AvarFEJ(1:2, 1:2) * [FEestJ(2) / FEestJ(1)^2, -1 / FEestJ(1)]');
            
            STDFEbetaY(r, dn, dt) = BetaYFEstd;
            SIZEFEbetaY(dn, dt) = SIZEFEbetaY(dn, dt) + 100 * (abs((BetaYFE - betaY) / BetaYFEstd) > 1.96) / R;
            SIZEFEphiY(dn, dt) = SIZEFEphiY(dn, dt) + 100 * (abs((FEest(1) - phiY) / sqrt(AvarFE(1, 1))) > 1.96) / R;
            SIZEFEbY(dn, dt) = SIZEFEbY(dn, dt) + 100 * (abs((FEest(2) - bY) / sqrt(AvarFE(2, 2))) > 1.96) / R;
            
            STDFEbetaYJ(r, dn, dt) = BetaYFEstdJ;
            SIZEFEbetaYJ(dn, dt) = SIZEFEbetaYJ(dn, dt) + 100 * (abs((BetaYFEJ - betaY) / BetaYFEstdJ) > 1.96) / R;
            SIZEFEphiYJ(dn, dt) = SIZEFEphiYJ(dn, dt) + 100 * (abs((FEestJ(1) - phiY) / sqrt(AvarFEJ(1, 1))) > 1.96) / R;
            SIZEFEbYJ(dn, dt) = SIZEFEbYJ(dn, dt) + 100 * (abs((FEestJ(2) - bY) / sqrt(AvarFEJ(2, 2))) > 1.96) / R;
            
        end
        
    end
    
    r
    toc
    
end

clearvars -except savefile g1Y g2Y lambdaY kappaX...
    COEFFEbetaY STDFEbetaY BIASFEbetaY MSEFEbetaY SIZEFEbetaY...
    COEFFEbetaYJ STDFEbetaYJ BIASFEbetaYJ MSEFEbetaYJ SIZEFEbetaYJ...
    COEFFEphiY STDFEphiY BIASFEphiY MSEFEphiY SIZEFEphiY...
    COEFFEphiYJ STDFEphiYJ BIASFEphiYJ MSEFEphiYJ SIZEFEphiYJ...
    COEFFEbY STDFEbY BIASFEbY MSEFEbY SIZEFEbY...
    COEFFEbYJ STDFEbYJ BIASFEbYJ MSEFEbYJ SIZEFEbYJ

RMSEFEbetaY = MSEFEbetaY.^0.5;
RMSEFEbetaYJ = MSEFEbetaYJ.^0.5;

RMSEFEphiY = MSEFEphiY.^0.5;
RMSEFEphiYJ = MSEFEphiYJ.^0.5;

RMSEFEbY = MSEFEbY.^0.5;
RMSEFEbYJ = MSEFEbYJ.^0.5;

toc

g1Y
g2Y
lambdaY
kappaX

BIASFEbetaY
BIASFEbetaYJ

RMSEFEbetaY
RMSEFEbetaYJ

SIZEFEbetaY
SIZEFEbetaYJ

toc