% Chudik, Pesaran and Yang (2016)
% 
% Data generating process
% 
% Y(i,t) = muY(i) + delta(t) + lambdaY * Y(i,t-1) 
%           + [1 - lambdaY] * beta * X(i,t) + U(i,t).
% X(i,t) = [1 - lambdaX] * muX(i) + [1 - lambdaX] * kappaX * Y(i,t-1)
%           + lambdaX * X(i,t-1) + v(i,t)
%
% beta = 0.5, lambdaX = 0.25, delta(t) = 0, 0.025 * t or 0.025 * t - 0.001 * t^2
% muX(i) ~ i.i.d. N(1, 1), muY(i) = muX(i) + etaY(i), etaY(i) ~ i.i.d. N(1, 1),
% v(i,t) ~ i.i.d. N(0, sigmav^2(i)), sigmav^2(i) = 0.5 + 0.25 * etav^2(i),
%           etav^2(i) ~ i.i.d. chi^2(2),
% u(i,t) ~ i.i.d. N(0, sigmau^2(i)), sigmau^2(i) = 0.5 + 0.25 * etau^2(i),
%           etau^2(i) ~ i.i.d. chi^2(2),
% lambdaY = 0.4 or 0.8, 
% kappaX = 0, 0.2 or 0.4.

clc
clear
close all

tic

% Replications
R = 2000;

% Parameters
betaY = 0.5;
lambdaX = 0.25;

g1Y = 0.025;
g2Y = 0;
lambdaY = 0.4;
kappaX = 0.2;

% phiY and bY

phiY = lambdaY - 1;
bY = (1 - lambdaY ) * betaY;

% Sample Size
NN = [30; 60; 100; 200; 500; 1000];
TT = [30; 60; 100; 200];

DN = length(NN);
DT = length(TT);

maxN = 1000;
maxT = 200;

% Burn-in
B = 100;

% Results

COEFFETEbetaY1 = zeros(R, DN, DT);
STDFETEbetaY1 = zeros(R, DN, DT);
BIASFETEbetaY1 = zeros(DN, DT);
MSEFETEbetaY1 = zeros(DN, DT);
SIZEFETEbetaY1 = zeros(DN, DT);

COEFFETEbetaYJ1 = zeros(R, DN, DT);
STDFETEbetaYJ1 = zeros(R, DN, DT);
BIASFETEbetaYJ1 = zeros(DN, DT);
MSEFETEbetaYJ1 = zeros(DN, DT);
SIZEFETEbetaYJ1 = zeros(DN, DT);

COEFFETEphiY1 = zeros(R, DN, DT);
STDFETEphiY1 = zeros(R, DN, DT);
BIASFETEphiY1 = zeros(DN, DT);
MSEFETEphiY1 = zeros(DN, DT);
SIZEFETEphiY1 = zeros(DN, DT);

COEFFETEphiYJ1 = zeros(R, DN, DT);
STDFETEphiYJ1 = zeros(R, DN, DT);
BIASFETEphiYJ1 = zeros(DN, DT);
MSEFETEphiYJ1 = zeros(DN, DT);
SIZEFETEphiYJ1 = zeros(DN, DT);

COEFFETEbY1 = zeros(R, DN, DT);
STDFETEbY1 = zeros(R, DN, DT);
BIASFETEbY1 = zeros(DN, DT);
MSEFETEbY1 = zeros(DN, DT);
SIZEFETEbY1 = zeros(DN, DT);

COEFFETEbYJ1 = zeros(R, DN, DT);
STDFETEbYJ1 = zeros(R, DN, DT);
BIASFETEbYJ1 = zeros(DN, DT);
MSEFETEbYJ1 = zeros(DN, DT);
SIZEFETEbYJ1 = zeros(DN, DT);

for r = 1:R
    
    N = maxN;
    T = B + maxT;
    
    % Parameters
    
    LambdaY = ones(N, 1) * lambdaY;
    BetaY = ones(N, 1) * betaY;
    G1Y = ones(N, 1) * g1Y;
    G2Y = ones(N, 1) * g2Y;
    
    SigmaU2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaU = SigmaU2.^0.5;
    
    LambdaX = ones(N, 1) * lambdaX;
    KappaX = ones(N, 1) * kappaX;
    
    CX = ones(N, 1) + randn(N, 1);
    CY = CX + ones(N, 1) + randn(N, 1);
    
    SigmaV2 = ones(N, 1) * 0.5 + (randn(N, 1).^2 + randn(N, 1).^2) * 0.25;
    SigmaV = SigmaV2.^0.5;
    
    % Errors
    
    UY = randn(N, T) .* (SigmaU * ones(1, T));
    VX = randn(N, T) .* (SigmaV * ones(1, T));
    
    % Generating Data
    
    LX = zeros(N, 1);
    LY = zeros(N, 1);
    
    XX = zeros(N, T);
    YY = zeros(N, T);

    for t = 1:T

        X = (ones(N, 1) - LambdaX) .* CX + LambdaX .* LX...
            + (ones(N, 1) - LambdaX) .* KappaX .* LY...
            + VX(:, t);

        Y = CY + G1Y * (t-B) + G2Y * (t-B)^2 ...
            + LambdaY .* LY + (ones(N, 1) - LambdaY) .* BetaY .* X + UY(:, t);

        XX(:, t) = X;
        YY(:, t) = Y;

        LX = X;
        LY = Y;

    end
    
    LYY = YY(:, B:(T-1));
    YY = YY(:, (B+1):T);
    DYY = YY - LYY;
    XX = XX(:, (B+1):T);
    
    % Estimation
    
    for dn = 1:DN
        
        N = NN(dn);
        
        for dt = 1:DT
            
            T = TT(dt);
            
            % Data
            
            Y = YY(1:N, 1:T);
            LY = LYY(1:N, 1:T);
            DY = DYY(1:N, 1:T);
            X = XX(1:N, 1:T);
            
            UnBd = ones(N, T);
            UnBd(1:floor(N/4), 1:floor(T/5)) = zeros(floor(N/4), floor(T/5));
            UnBd(1:floor(N/4), ceil(4*T/5)+1:T) = zeros(floor(N/4), floor(T/5));
            UnBd(floor(N/4)+1:floor(N/2), 1:floor(T/3)) = zeros(floor(N/2)-floor(N/4), floor(T/3));
            UnBd(floor(N/4)+1:floor(N/2), ceil(2*T/3)+1:T) = zeros(floor(N/2)-floor(N/4), floor(T/3));
            
            Y = Y .* UnBd;
            LY = LY .* UnBd;
            DY = DY .* UnBd;
            X = X .* UnBd;
            
            Ya = Y(:, 1:T/2);
            LYa = LY(:, 1:T/2);
            DYa = DY(:, 1:T/2);
            Xa = X(:, 1:T/2);
            
            Yb = Y(:, T/2+1:T);
            LYb = LY(:, T/2+1:T);
            DYb = DY(:, T/2+1:T);
            Xb = X(:, T/2+1:T);
            
            % Transform data to long vectors
            
            Y = Y';
            LY = LY';
            DY = DY';
            X = X';
            LongY = Y(Y~=0);
            LongLY = LY(LY~=0);
            LongDY = DY(DY~=0);
            LongX = X(X~=0);
            Y = Y';
            LY = LY';
            DY = DY';
            X = X';
            
            Ya = Ya';
            LYa = LYa';
            DYa = DYa';
            Xa = Xa';
            LongYa = Ya(Ya~=0);
            LongLYa = LYa(LYa~=0);
            LongDYa = DYa(DYa~=0);
            LongXa = Xa(Xa~=0);
            Ya = Ya';
            LYa = LYa';
            DYa = DYa';
            Xa = Xa';
            
            Yb = Yb';
            LYb = LYb';
            DYb = DYb';
            Xb = Xb';
            LongYb = Yb(Yb~=0);
            LongLYb = LYb(LYb~=0);
            LongDYb = DYb(DYb~=0);
            LongXb = Xb(Xb~=0);
            Yb = Yb';
            LYb = LYb';
            DYb = DYb';
            Xb = Xb';
            
            % Full FE-TE
            
            z1 = eye(N);
            z1 = z1(1:floor(N/4), :);
            Z1 = kron(z1, ones(ceil(4*T/5)-floor(T/5), 1));
            z1 = eye(N);
            z1 = z1(floor(N/4)+1:floor(N/2), :);
            Z1 = [Z1; kron(z1, ones(ceil(2*T/3)-floor(T/3), 1))];
            z1 = eye(N);
            z1 = z1(floor(N/2)+1:N, :);
            Z1 = [Z1; kron(z1, ones(T, 1))];
            z2 = eye(T);
            z2 = z2(floor(T/5)+1:ceil(4*T/5), :);
            Z2 = kron(ones(floor(N/4), 1), z2);
            z2 = eye(T);
            z2 = z2(floor(T/3)+1:ceil(2*T/3), :);
            Z2 = [Z2; kron(ones(floor(N/2)-floor(N/4), 1), z2)];
            Z2 = [Z2; kron(ones(N-floor(N/2), 1), eye(T))];
            Z = [Z1, Z2];
            ZLongY = Z' * LongY;
            ZLongLY = Z' * LongLY;
            ZLongDY = Z' * LongDY;
            ZLongX = Z' * LongX;
            Q = Z' * Z;
            Q = pinv(Q);
            ZQ = Z * Q;
            LongY1 = LongY - ZQ * ZLongY;
            LongLY1 = LongLY - ZQ * ZLongLY;
            LongDY1 = LongDY - ZQ * ZLongDY;
            LongX1 = LongX - ZQ * ZLongX;
            
            clear ZlongY ZLongLY ZLongDY ZLongX Z Q ZQ z1 z2 Z1 Z2
            
            z1 = eye(N);
            z1 = z1(1:floor(N/4), :);
            Z1a = kron(z1, ones(T/2-floor(T/5), 1));
            z1 = eye(N);
            z1 = z1(floor(N/4)+1:floor(N/2), :);
            Z1a = [Z1a; kron(z1, ones(T/2-floor(T/3), 1))];
            z1 = eye(N);
            z1 = z1(floor(N/2)+1:N, :);
            Z1a = [Z1a; kron(z1, ones(T/2, 1))];
            z2 = eye(T/2);
            z2 = z2(floor(T/5)+1:T/2, :);
            Z2a = kron(ones(floor(N/4), 1), z2);
            z2 = eye(T/2);
            z2 = z2(floor(T/3)+1:T/2, :);
            Z2a = [Z2a; kron(ones(floor(N/2)-floor(N/4), 1), z2)];
            Z2a = [Z2a; kron(ones(N-floor(N/2), 1), eye(T/2))];
            Za = [Z1a, Z2a];
            ZaLongYa = Za' * LongYa;
            ZaLongLYa = Za' * LongLYa;
            ZaLongDYa = Za' * LongDYa;
            ZaLongXa = Za' * LongXa;
            Qa = Za' * Za;
            Qa = pinv(Qa);
            ZaQa = Za * Qa;
            LongY1a = LongYa - ZaQa * ZaLongYa;
            LongLY1a = LongLYa - ZaQa * ZaLongLYa;
            LongDY1a = LongDYa - ZaQa * ZaLongDYa;
            LongX1a = LongXa - ZaQa * ZaLongXa;
            
            clear ZaLongYa ZaLongLYa ZaLongDYa ZaLongXa Za Qa ZaQa z1 z2 Z1a Z2a
            
            z1 = eye(N);
            z1 = z1(1:floor(N/4), :);
            Z1b = kron(z1, ones(ceil(4*T/5)-T/2, 1));
            z1 = eye(N);
            z1 = z1(floor(N/4)+1:floor(N/2), :);
            Z1b = [Z1b; kron(z1, ones(ceil(2*T/3)-T/2, 1))];
            z1 = eye(N);
            z1 = z1(floor(N/2)+1:N, :);
            Z1b = [Z1b; kron(z1, ones(T/2, 1))];
            z2 = eye(T/2);
            z2 = z2(1:ceil(4*T/5)-T/2, :);
            Z2b = kron(ones(floor(N/4), 1), z2);
            z2 = eye(T/2);
            z2 = z2(1:ceil(2*T/3)-T/2, :);
            Z2b = [Z2b; kron(ones(floor(N/2)-floor(N/4), 1), z2)];
            Z2b = [Z2b; kron(ones(N-floor(N/2), 1), eye(T/2))];
            Zb = [Z1b, Z2b];
            ZbLongYb = Zb' * LongYb;
            ZbLongLYb = Zb' * LongLYb;
            ZbLongDYb = Zb' * LongDYb;
            ZbLongXb = Zb' * LongXb;
            Qb = Zb' * Zb;
            Qb = pinv(Qb);
            ZbQb = Zb * Qb;
            LongY1b = LongYb - ZbQb * ZbLongYb;
            LongLY1b = LongLYb - ZbQb * ZbLongLYb;
            LongDY1b = LongDYb - ZbQb * ZbLongDYb;
            LongX1b = LongXb - ZbQb * ZbLongXb;
            
            clear ZbLongYb ZbLongLYb ZbLongDYb ZbLongXb Zb Qb ZbQb z1 z2 Z1b Z2b
            
            % Back to matrices
            
            Y1 = zeros(N, T);
            LY1 = zeros(N, T);
            DY1 = zeros(N, T);
            X1 = zeros(N, T);
            Y1a = zeros(N, T/2);
            LY1a = zeros(N, T/2);
            DY1a = zeros(N, T/2);
            X1a = zeros(N, T/2);
            Y1b = zeros(N, T/2);
            LY1b = zeros(N, T/2);
            DY1b = zeros(N, T/2);
            X1b = zeros(N, T/2);
            
            k1 = 1;
            k1a = 1;
            k1b = 1;
            k2 = floor(N/4)*(ceil(4*T/5)-floor(T/5));
            k2a = floor(N/4)*(T/2-floor(T/5));
            k2b = floor(N/4)*(ceil(4*T/5)-T/2);
            
            Y1(1:floor(N/4), floor(T/5)+1:ceil(4*T/5)) = reshape(LongY1(k1:k2), ceil(4*T/5)-floor(T/5), [])';
            LY1(1:floor(N/4), floor(T/5)+1:ceil(4*T/5)) = reshape(LongLY1(k1:k2), ceil(4*T/5)-floor(T/5), [])';
            DY1(1:floor(N/4), floor(T/5)+1:ceil(4*T/5)) = reshape(LongDY1(k1:k2), ceil(4*T/5)-floor(T/5), [])';
            X1(1:floor(N/4), floor(T/5)+1:ceil(4*T/5)) = reshape(LongX1(k1:k2), ceil(4*T/5)-floor(T/5), [])';
            Y1a(1:floor(N/4), floor(T/5)+1:T/2) = reshape(LongY1a(k1a:k2a), T/2-floor(T/5), [])';
            LY1a(1:floor(N/4), floor(T/5)+1:T/2) = reshape(LongLY1a(k1a:k2a), T/2-floor(T/5), [])';
            DY1a(1:floor(N/4), floor(T/5)+1:T/2) = reshape(LongDY1a(k1a:k2a), T/2-floor(T/5), [])';
            X1a(1:floor(N/4), floor(T/5)+1:T/2) = reshape(LongX1a(k1a:k2a), T/2-floor(T/5), [])';
            Y1b(1:floor(N/4), 1:ceil(4*T/5)-T/2) = reshape(LongY1b(k1b:k2b), ceil(4*T/5)-T/2, [])';
            LY1b(1:floor(N/4), 1:ceil(4*T/5)-T/2) = reshape(LongLY1b(k1b:k2b), ceil(4*T/5)-T/2, [])';
            DY1b(1:floor(N/4), 1:ceil(4*T/5)-T/2) = reshape(LongDY1b(k1b:k2b), ceil(4*T/5)-T/2, [])';
            X1b(1:floor(N/4), 1:ceil(4*T/5)-T/2) = reshape(LongX1b(k1b:k2b), ceil(4*T/5)-T/2, [])';
            
            k1 = k2 + 1;
            k1a = k2a + 1;
            k1b = k2b + 1;
            k2 = k2 + (floor(N/2)-floor(N/4))*(ceil(2*T/3)-floor(T/3));
            k2a = k2a + (floor(N/2)-floor(N/4))*(T/2-floor(T/3));
            k2b = k2b + (floor(N/2)-floor(N/4))*(ceil(2*T/3)-T/2);
            
            Y1(floor(N/4)+1:floor(N/2), floor(T/3)+1:ceil(2*T/3)) = reshape(LongY1(k1:k2), ceil(2*T/3)-floor(T/3), [])';
            LY1(floor(N/4)+1:floor(N/2), floor(T/3)+1:ceil(2*T/3)) = reshape(LongLY1(k1:k2), ceil(2*T/3)-floor(T/3), [])';
            DY1(floor(N/4)+1:floor(N/2), floor(T/3)+1:ceil(2*T/3)) = reshape(LongDY1(k1:k2), ceil(2*T/3)-floor(T/3), [])';
            X1(floor(N/4)+1:floor(N/2), floor(T/3)+1:ceil(2*T/3)) = reshape(LongX1(k1:k2), ceil(2*T/3)-floor(T/3), [])';
            Y1a(floor(N/4)+1:floor(N/2), floor(T/3)+1:T/2) = reshape(LongY1a(k1a:k2a), T/2-floor(T/3), [])';
            LY1a(floor(N/4)+1:floor(N/2), floor(T/3)+1:T/2) = reshape(LongLY1a(k1a:k2a), T/2-floor(T/3), [])';
            DY1a(floor(N/4)+1:floor(N/2), floor(T/3)+1:T/2) = reshape(LongDY1a(k1a:k2a), T/2-floor(T/3), [])';
            X1a(floor(N/4)+1:floor(N/2), floor(T/3)+1:T/2) = reshape(LongX1a(k1a:k2a), T/2-floor(T/3), [])';
            Y1b(floor(N/4)+1:floor(N/2), 1:ceil(2*T/3)-T/2) = reshape(LongY1b(k1b:k2b), ceil(2*T/3)-T/2, [])';
            LY1b(floor(N/4)+1:floor(N/2), 1:ceil(2*T/3)-T/2) = reshape(LongLY1b(k1b:k2b), ceil(2*T/3)-T/2, [])';
            DY1b(floor(N/4)+1:floor(N/2), 1:ceil(2*T/3)-T/2) = reshape(LongDY1b(k1b:k2b), ceil(2*T/3)-T/2, [])';
            X1b(floor(N/4)+1:floor(N/2), 1:ceil(2*T/3)-T/2) = reshape(LongX1b(k1b:k2b), ceil(2*T/3)-T/2, [])';
            
            k1 = k2 + 1;
            k1a = k2a + 1;
            k1b = k2b + 1;
            k2 = k2 + (N-floor(N/2))*T;
            k2a = k2a + (N-floor(N/2))*T/2;
            k2b = k2b + (N-floor(N/2))*T/2;
            
            Y1(floor(N/2)+1:N, :) = reshape(LongY1(k1:k2), T, [])';
            LY1(floor(N/2)+1:N, :) = reshape(LongLY1(k1:k2), T, [])';
            DY1(floor(N/2)+1:N, :) = reshape(LongDY1(k1:k2), T, [])';
            X1(floor(N/2)+1:N, :) = reshape(LongX1(k1:k2), T, [])';
            Y1a(floor(N/2)+1:N, :) = reshape(LongY1a(k1a:k2a), T/2, [])';
            LY1a(floor(N/2)+1:N, :) = reshape(LongLY1a(k1a:k2a), T/2, [])';
            DY1a(floor(N/2)+1:N, :) = reshape(LongDY1a(k1a:k2a), T/2, [])';
            X1a(floor(N/2)+1:N, :) = reshape(LongX1a(k1a:k2a), T/2, [])';
            Y1b(floor(N/2)+1:N, :) = reshape(LongY1b(k1b:k2b), T/2, [])';
            LY1b(floor(N/2)+1:N, :) = reshape(LongLY1b(k1b:k2b), T/2, [])';
            DY1b(floor(N/2)+1:N, :) = reshape(LongDY1b(k1b:k2b), T/2, [])';
            X1b(floor(N/2)+1:N, :) = reshape(LongX1b(k1b:k2b), T/2, [])';
            
            clear LongY1 LongLY1 LongDY1 LongX1 LongY1a LongLY1a LongDY1a LongX1a LongY1b LongLY1b LongDY1b LongX1b
            
            Y1ab = [Y1a, Y1b];
            LY1ab = [LY1a, LY1b];
            DY1ab = [DY1a, DY1b];
            X1ab = [X1a, X1b];
            
            % FE Estimation
            
            WMW1 = zeros(2, 2);
            WMDY1 = zeros(2, 1);
            WMW1a = zeros(2, 2);
            WMDY1a = zeros(2, 1);
            WMW1b = zeros(2, 2);
            WMDY1b = zeros(2, 1);
            
            for i = 1:N
                
                dYm1 = DY1(i, :)';
                Xm1 = X1(i, :)';
                lYm1 = LY1(i, :)';
                Wm1 = [lYm1, Xm1];
                WMW1 = WMW1 + Wm1' * Wm1;
                WMDY1 = WMDY1 + Wm1' * dYm1;
                dYm1a = DY1a(i, :)';
                Xm1a = X1a(i, :)';
                lYm1a = LY1a(i, :)';
                Wm1a = [lYm1a, Xm1a];
                WMW1a = WMW1a + Wm1a' * Wm1a;
                WMDY1a = WMDY1a + Wm1a' * dYm1a;
                dYm1b = DY1b(i, :)';
                Xm1b = X1b(i, :)';
                lYm1b = LY1b(i, :)';
                Wm1b = [lYm1b, Xm1b];
                WMW1b = WMW1b + Wm1b' * Wm1b;
                WMDY1b = WMDY1b + Wm1b' * dYm1b;
                
            end
            
            invWMW1 = WMW1^(-1);
            FETEest1 = invWMW1 * WMDY1;
            BetaYFETE1 = - FETEest1(2) / FETEest1(1);
            invWMW1a = WMW1a^(-1);
            FETEest1a = invWMW1a * WMDY1a;
            invWMW1b = WMW1b^(-1);
            FETEest1b = invWMW1b * WMDY1b;
            FETEestJ1 = 2 * FETEest1 - 0.5 * FETEest1a - 0.5 * FETEest1b;
            BetaYFETEJ1 = - FETEestJ1(2) / FETEestJ1(1);
            
            COEFFETEbetaY1(r, dn, dt) = BetaYFETE1;
            BIASFETEbetaY1(dn, dt) = BIASFETEbetaY1(dn, dt) + 100 * (BetaYFETE1 - betaY) / R;
            MSEFETEbetaY1(dn, dt) = MSEFETEbetaY1(dn, dt) + 10000 * (BetaYFETE1 - betaY)^2 / R;
            COEFFETEbetaYJ1(r, dn, dt) = BetaYFETEJ1;
            BIASFETEbetaYJ1(dn, dt) = BIASFETEbetaYJ1(dn, dt) + 100 * (BetaYFETEJ1 - betaY) / R;
            MSEFETEbetaYJ1(dn, dt) = MSEFETEbetaYJ1(dn, dt) + 10000 * (BetaYFETEJ1 - betaY)^2 / R;
            
            COEFFETEphiY1(r, dn, dt) = FETEest1(1);
            BIASFETEphiY1(dn, dt) = BIASFETEphiY1(dn, dt) + 100 * (FETEest1(1) - phiY) / R;
            MSEFETEphiY1(dn, dt) = MSEFETEphiY1(dn, dt) + 10000 * (FETEest1(1) - phiY)^2 / R;
            COEFFETEphiYJ1(r, dn, dt) = FETEestJ1(1);
            BIASFETEphiYJ1(dn, dt) = BIASFETEphiYJ1(dn, dt) + 100 * (FETEestJ1(1) - phiY) / R;
            MSEFETEphiYJ1(dn, dt) = MSEFETEphiYJ1(dn, dt) + 10000 * (FETEestJ1(1) - phiY)^2 / R;
            
            COEFFETEbY1(r, dn, dt) = FETEest1(2);
            BIASFETEbY1(dn, dt) = BIASFETEbY1(dn, dt) + 100 * (FETEest1(2) - bY) / R;
            MSEFETEbY1(dn, dt) = MSEFETEbY1(dn, dt) + 10000 * (FETEest1(2) - bY)^2 / R;
            COEFFETEbYJ1(r, dn, dt) = FETEestJ1(2);
            BIASFETEbYJ1(dn, dt) = BIASFETEbYJ1(dn, dt) + 100 * (FETEestJ1(2) - bY) / R;
            MSEFETEbYJ1(dn, dt) = MSEFETEbYJ1(dn, dt) + 10000 * (FETEestJ1(2) - bY)^2 / R;
            
            % Variance
            
            WuuW1 = zeros(2, 2);
            WuuWJ1 = zeros(2, 2);
            
            for i = 1:N
                
                dYm1 = DY1(i, :)';
                Xm1 = X1(i, :)';
                lYm1 = LY1(i, :)';
                Wm1 = [lYm1, Xm1];
                u1 = dYm1 - Wm1 * FETEest1;
                Xmm1 = 2 * X1(i, :)' - X1ab(i, :)';
                lYmm1 = 2 * LY1(i, :)' - LY1ab(i, :)';
                Wmm1 = [lYmm1, Xmm1];
                uJ1 = dYm1 - Wm1 * FETEestJ1;
                
                for t = 1:T
                    
                    WuuW1 = WuuW1 + u1(t)^2 * Wm1(t, :)' * Wm1(t, :);
                    WuuWJ1 = WuuWJ1 + uJ1(t)^2 * Wmm1(t, :)' * Wmm1(t, :);
                    
                end
                
            end
            
            AvarFETE1 = invWMW1 * WuuW1 * invWMW1;
            BetaYFETEstd1 = sqrt([FETEest1(2)/FETEest1(1)^2, -1/FETEest1(1)] * AvarFETE1(1:2, 1:2) * [FETEest1(2)/FETEest1(1)^2; -1/FETEest1(1)]);
            AvarFETEJ1 = invWMW1 * WuuWJ1 * invWMW1;
            BetaYFETEstdJ1 = sqrt([FETEestJ1(2)/FETEestJ1(1)^2, -1/FETEestJ1(1)] * AvarFETEJ1(1:2, 1:2) * [FETEestJ1(2)/FETEestJ1(1)^2; -1/FETEestJ1(1)]);
            
            STDFETEbetaY1(r, dn, dt) = BetaYFETEstd1;
            SIZEFETEbetaY1(dn, dt) = SIZEFETEbetaY1(dn, dt) + 100 * (abs((BetaYFETE1 - betaY) / BetaYFETEstd1) > 1.96) / R;
            STDFETEbetaYJ1(r, dn, dt) = BetaYFETEstdJ1;
            SIZEFETEbetaYJ1(dn, dt) = SIZEFETEbetaYJ1(dn, dt) + 100 * (abs((BetaYFETEJ1 - betaY) / BetaYFETEstdJ1) > 1.96) / R;
            
            STDFETEphiY1(r, dn, dt) = sqrt(AvarFETE1(1, 1));
            SIZEFETEphiY1(dn, dt) = SIZEFETEphiY1(dn, dt) + 100 * (abs((FETEest1(1) - phiY) / sqrt(AvarFETE1(1, 1))) > 1.96) / R;
            STDFETEphiYJ1(r, dn, dt) = sqrt(AvarFETEJ1(1, 1));
            SIZEFETEphiYJ1(dn, dt) = SIZEFETEphiYJ1(dn, dt) + 100 * (abs((FETEestJ1(1) - phiY) / sqrt(AvarFETEJ1(1, 1))) > 1.96) / R;
            
            STDFETEbY1(r, dn, dt) = sqrt(AvarFETE1(2, 2));
            SIZEFETEbY1(dn, dt) = SIZEFETEbY1(dn, dt) + 100 * (abs((FETEest1(2) - bY) / sqrt(AvarFETE1(2, 2))) > 1.96) / R;
            STDFETEbYJ1(r, dn, dt) = sqrt(AvarFETEJ1(2, 2));
            SIZEFETEbYJ1(dn, dt) = SIZEFETEbYJ1(dn, dt) + 100 * (abs((FETEestJ1(2) - bY) / sqrt(AvarFETEJ1(2, 2))) > 1.96) / R;
            
        end
        
    end
    
    r
    toc
    
end

clearvars -except savefile g1Y g2Y lambdaY kappaX...
    COEFFETEbetaY1 STDFETEbetaY1 BIASFETEbetaY1 MSEFETEbetaY1 SIZEFETEbetaY1...
    COEFFETEbetaYJ1 STDFETEbetaYJ1 BIASFETEbetaYJ1 MSEFETEbetaYJ1 SIZEFETEbetaYJ1...
    COEFFETEphiY1 STDFETEphiY1 BIASFETEphiY1 MSEFETEphiY1 SIZEFETEphiY1...
    COEFFETEphiYJ1 STDFETEphiYJ1 BIASFETEphiYJ1 MSEFETEphiYJ1 SIZEFETEphiYJ1...
    COEFFETEbY1 STDFETEbY1 BIASFETEbY1 MSEFETEbY1 SIZEFETEbY1...
    COEFFETEbYJ1 STDFETEbYJ1 BIASFETEbYJ1 MSEFETEbYJ1 SIZEFETEbYJ1

RMSEFETEbetaY1 = MSEFETEbetaY1.^0.5;
RMSEFETEbetaYJ1 = MSEFETEbetaYJ1.^0.5;

RMSEFETEphiY1 = MSEFETEphiY1.^0.5;
RMSEFETEphiYJ1 = MSEFETEphiYJ1.^0.5;

RMSEFETEbY1 = MSEFETEbY1.^0.5;
RMSEFETEbYJ1 = MSEFETEbYJ1.^0.5;

toc

g1Y
g2Y
lambdaY
kappaX

BIASFETEbetaY1
BIASFETEbetaYJ1

RMSEFETEbetaY1
RMSEFETEbetaYJ1

SIZEFETEbetaY1
SIZEFETEbetaYJ1

toc